package com.sitemesh;

import java.util.Properties;
import javax.servlet.Config;
import javax.servlet.http.HttpServletRequest;
import com.sitemesh.mapper.DefaultDecoratorMapper;
import com.sitemesh.parser.*;

/**
 * Factory responsible for creating appropriate instances of implementations.
 * This is a singleton and is obtained through {@link #getInstance(javax.servlet.Config)}.
 *
 * @author <a href="mailto:joe@truemesh.com">Joe Walnes</a>
 * @version $Revision: 1.7 $
 */
public class Factory {

	/**
	 * @label Singleton instance
	 */
	private static Factory instance;

	/**
	 * ServletConfig or FilterConfig.
	 */
	protected Config config;

	/**
	 * Instance of DecoratorMapper.
	 * Because it is thread-safe it can be shared by multiple clients.
	 * @label creates
	 */
	protected DecoratorMapper decoratorMapper;

	/**
	 * @directed
	 * @label creates suitable
	 */
	/*#private PageParser lnkPageParser; */

	/**
	 * Constructor for default implementation of Factory.
	 * Should never be called by client. Singleton instance should be
	 * obtained instead.
	 *
	 * @see #getInstance(javax.servlet.Config config)
	 */
	protected Factory(Config config) {
		this.config = config;
	}

	/**
	 * Entry-point for obtaining singleton instance of Factory.
	 */
	public static Factory getInstance(Config config) {
		// @todo: Static instance should acount for
		//        multiple configs using same factory.
		// @todo: Allow to return different implementations
		//        of Factory depending on a system/config
		//        property.
		if (instance == null) instance = new Factory(config);
		return instance;
	}

	/**
	 * Return instance of DecoratorMapper.
	 */
	public DecoratorMapper getDecoratorMapper() {
		// @todo: Allow to return different implementations
		//        of DecoratorMapper depending on a system/config
		//        property.
		if (decoratorMapper == null) {
			try {
				decoratorMapper = new DefaultDecoratorMapper();
				decoratorMapper.init(config, new Properties());
			}
			catch (InstantiationException e) {}
		}
		return decoratorMapper;
	}

	/**
	 * Create a PageParser suitable for the given content-type.
	 *
	 * <p>For example, if the supplied parameter is <code>text/html</code>
	 * a parser shall be returned that can parse HTML accordingly.</p> Never returns null.
	 *
	 * @param contentType The MIME content-type of the data to be parsed
	 * @return Appropriate <code>PageParser</code> for reading data
	 */
	public PageParser getPageParser(String contentType) {
		if ( contentType == null ) return defaultParser();
		else if ( contentType.equals( "text/html" ) ) return new DOMPageParser();
		else return defaultParser();
	}

	/**
	 * Default PageParser if no other can be determined.
	 */
	private PageParser defaultParser() {
		return new AbstractPageParser() {
			public Page parse( byte[] data ) {
				return new UnParsedPage( data );
			}
		};
	}

}
